#!/usr/bin/env python
# vim:fileencoding=utf-8:ts=4:sw=4:sta:et:sts=4:ai

__license__   = 'GPL v3'
__copyright__ = '2011, Karl Weckworth <kweckwor@gmail.com>'
__docformat__ = 'restructuredtext en'

from calibre_plugins.ManageSonyX50BookList import PLUGIN_NAME

def showInfoDialog(gui, message):
    from calibre.gui2 import info_dialog
    
    info_dialog(gui, PLUGIN_NAME, message, show=True, show_copy_button= False)

def showWarningDialog(gui, message):
    from calibre.gui2 import warning_dialog
    
    warning_dialog(gui, PLUGIN_NAME, message, show=True, show_copy_button= False)

def showErrorDialog(gui, message):
    from calibre.gui2 import error_dialog
    
    error_dialog(gui, PLUGIN_NAME, message, show=True, show_copy_button= False)

def createMenu(interfaceAction):
    try:
        from PyQt4.Qt import QMenu
    except ImportError:
        from PyQt5.Qt import QMenu
    
    return QMenu(interfaceAction.gui)

def createAction(interfaceAction, parentMenu, actionTitle, actionTooltip, actionIcon=None, actionTrigger=None):
    action = interfaceAction.create_action(spec=(actionTitle, None, actionTooltip, None), attr=actionTitle)
    
    if actionIcon is not None:
        action.setIcon(actionIcon)
    
    if actionTrigger is not None:
        action.triggered.connect(actionTrigger)
        
    parentMenu.addAction(action)
    
    return action

def createPixmap(iconName=None, iconResource=None):
    try:
        from PyQt4.Qt import QPixmap
    except ImportError:
        from PyQt5.Qt import QPixmap
    
    pixmap = None
    
    if iconName is not None:
        pixmap = QPixmap(I(iconName))
    
    elif iconResource is not None:
        pixmap = QPixmap()
        pixmap.loadFromData(iconResource)
    
    return pixmap
    
def createIcon(iconName=None, iconResource=None):
    try:
        from PyQt4.Qt import QIcon, QPixmap
    except ImportError:
        from PyQt5.Qt import QIcon, QPixmap
    
    icon = None
    
    if iconName is not None:
        icon = QIcon(I(iconName))
    
    elif iconResource is not None:
        pixmap = QPixmap()
        pixmap.loadFromData(iconResource)
        icon = QIcon(pixmap)
    
    return icon
    
def getUniqueName(name, existingNames):
    namePrefix = name
    tryName = name
    index = 1
        
    while tryName in existingNames:
        tryName = namePrefix + ':' + str(index)
        index = index + 1
        
    return tryName
    
def getUnicode(value, encoding='utf-8'):
    if isinstance(value, unicode):
        returnValue = value
    else:
        returnValue = unicode(value, encoding)
        
    return returnValue
 
CONFIG_LABEL_BOOKS_LIST_SELECT_TITLE = _('Set Books List')
CONFIG_LABEL_UNREAD_LIST_SELECT_TITLE = _('Set Unread List')
CONFIG_LABEL_BOOKS_LIST_TAB = _('Books List')
CONFIG_LABEL_UNREAD_LIST_TAB = _('Unread List')
CONFIG_LABEL_BOOKS_LIST = _('The Books List profile configuration determines how books for the Books List are chosen and how thay will appear on your reader.')
CONFIG_LABEL_UNREAD_LIST = _('The Unread List profile configuration determines how books for the Unread List are chosen.')
CONFIG_LABEL_CHOOSE_PROFILE = _('Choose Profile:')
CONFIG_LABEL_PROFILE = _('Profile:')
CONFIG_LABEL_PROFILES = _('Profiles:')
CONFIG_LABEL_DETAILS = _('Details:')
CONFIG_LABEL_SELECTION = _('Choose books from Device/Card A/Card B book list selections.')
CONFIG_LABEL_COLLECTIONS = _('Choose books from device collections: ')
CONFIG_LABEL_QUERY = _('Choose books using library query: ')
CONFIG_LABEL_CHOOSE_ONE = _('Choose one book from each source.')
CONFIG_LABEL_CHOOSE_NEWEST = _('Choose books by newest first.')
CONFIG_LABEL_CHOOSE_OLDEST = _('Choose books by oldest first.')
CONFIG_LABEL_CHOOSE_RANDOM = _('Choose books randomly.')
CONFIG_LABEL_ORDER_DATE = _('Order books list by date.')
CONFIG_LABEL_ORDER_AUTHOR = _('Order books list by author.')
CONFIG_LABEL_ORDER_TITLE = _('Order books list by title.')
CONFIG_LABEL_CHOOSE_GROUP = _('Choose Books From:')
CONFIG_LABEL_SELECTION_OPTION = _('Device/A/B book list selections')
CONFIG_LABEL_QUERY_OPTION = _('Library Query:')
CONFIG_LABEL_COLLECTIONS_OPTION = _('Device Collections:')
CONFIG_LABEL_STRATEGY_GROUP = _('Choose Books By:')
CONFIG_LABEL_ONE_OPTION = _('One book from each source')
CONFIG_LABEL_NEWEST_OPTION = _('Newest first')
CONFIG_LABEL_OLDEST_OPTION = _('Oldest first')
CONFIG_LABEL_RANDOM_OPTION = _('Randomly')
CONFIG_LABEL_ORDER_GROUP = _('Order Books List By:')
CONFIG_LABEL_DATE_OPTION = _('Date')
CONFIG_LABEL_AUTHOR_OPTION = _('Author')
CONFIG_LABEL_TITLE_OPTION = _('Title')
CONFIG_LABEL_ADD = _('Add')
CONFIG_LABEL_DELETE = _('Delete')
CONFIG_LABEL_SHOW_WARNING = _('Show warning when setting Unread List.')
CONFIG_LABEL_UNREAD_WARNING_TITLE = _('Are you sure?')
CONFIG_LABEL_UNREAD_WARNING = _('<b>WARNING</b>: Proceeding with this action will <b>reset the books</b> contained in the Unread Books collection based on the selected profile configuration. Are you sure?')
CONFIG_LABEL_ALWAYS_SHOW_WARNING = _('Always show this warning (enable again through plugin customization)')

DEFAULT_PROFILE_NAME = _('New Profile')
DEFAULT_COLLECTION_NAME = _('New Collection')

ICON_WARNING = 'dialog_warning.png'

from calibre_plugins.ManageSonyX50BookList.action import CONFIG_BOOKS_LIST_KEY
from calibre_plugins.ManageSonyX50BookList.action import CONFIG_UNREAD_LIST_KEY
from calibre_plugins.ManageSonyX50BookList.action import CONFIG_PROFILES_KEY
from calibre_plugins.ManageSonyX50BookList.action import CONFIG_NAME_KEY
from calibre_plugins.ManageSonyX50BookList.action import CONFIG_BOOKS_KEY
from calibre_plugins.ManageSonyX50BookList.action import CONFIG_BOOKS_VALUE_SELECTION
from calibre_plugins.ManageSonyX50BookList.action import CONFIG_BOOKS_COLLECTIONS_KEY
from calibre_plugins.ManageSonyX50BookList.action import CONFIG_BOOKS_QUERY_KEY
from calibre_plugins.ManageSonyX50BookList.action import CONFIG_STRATEGY_KEY
from calibre_plugins.ManageSonyX50BookList.action import CONFIG_STRATEGY_VALUE_ONE
from calibre_plugins.ManageSonyX50BookList.action import CONFIG_STRATEGY_VALUE_NEWEST
from calibre_plugins.ManageSonyX50BookList.action import CONFIG_STRATEGY_VALUE_OLDEST
from calibre_plugins.ManageSonyX50BookList.action import CONFIG_STRATEGY_VALUE_RANDOM
from calibre_plugins.ManageSonyX50BookList.action import CONFIG_ORDER_KEY
from calibre_plugins.ManageSonyX50BookList.action import CONFIG_ORDER_VALUE_DATE
from calibre_plugins.ManageSonyX50BookList.action import CONFIG_ORDER_VALUE_AUTHOR
from calibre_plugins.ManageSonyX50BookList.action import CONFIG_ORDER_VALUE_TITLE
from calibre_plugins.ManageSonyX50BookList.action import CONFIG_POS_KEY
from calibre_plugins.ManageSonyX50BookList.action import CONFIG_SHOW_WARNING_KEY

from calibre_plugins.ManageSonyX50BookList.action import DEFAULT_CONFIG_BOOKS_LIST_QUERY
from calibre_plugins.ManageSonyX50BookList.action import DEFAULT_CONFIG_UNREAD_LIST_QUERY

try:
    from PyQt4.Qt import QDialog
except ImportError:
    from PyQt5.Qt import QDialog

class ManageBookListView(QDialog):
    def __init__(self, parent):
        try:
            from PyQt4 import QtCore
        except ImportError:
            from PyQt5 import Qt as QtCore
            
        try:
            from PyQt4.Qt import QGridLayout, QLabel, QDialogButtonBox
        except ImportError:
            from PyQt5.Qt import QGridLayout, QLabel, QDialogButtonBox
        
        QDialog.__init__(self, parent)
        
        self.setWindowTitle(CONFIG_LABEL_BOOKS_LIST_SELECT_TITLE)
        
        self.dialogLayout = QGridLayout(self)
        
        self.setBookListText = QLabel(CONFIG_LABEL_BOOKS_LIST)
        self.setBookListText.setWordWrap(True)
        self.dialogLayout.addWidget(self.setBookListText, 0, 0, 1, 2)

        self.profileLabel = QLabel()
        self.dialogLayout.addWidget(self.profileLabel, 1, 0, 1, 1, QtCore.Qt.AlignLeft)
                
        self.infoLabel = QLabel('<b>' + CONFIG_LABEL_DETAILS + '</b>')
        self.dialogLayout.addWidget(self.infoLabel, 2, 0, 1, 2, QtCore.Qt.AlignLeft | QtCore.Qt.AlignTop)

        self.profileDetails = QLabel()
        self.profileDetails.setWordWrap(True)
        self.dialogLayout.addWidget(self.profileDetails, 3, 0, 1, 2)
        
        self.buttonBox = QDialogButtonBox(QDialogButtonBox.Ok | QDialogButtonBox.Cancel)
        self.dialogLayout.addWidget(self.buttonBox, 4, 0, 1, 2)
        
        self.dialogLayout.setColumnStretch(1, 1)
        self.dialogLayout.setColumnMinimumWidth(1, 200)
    
    def getProfileName(self):
        try:
            from PyQt4.Qt import QComboBox
        except ImportError:
            from PyQt5.Qt import QComboBox
        
        if isinstance(self.profileValue, QComboBox):
            profileName = getUnicode(self.profileValue.itemText(self.profileValue.currentIndex()))
        else:
            profileName = getUnicode(self.profileValue.text())
            
        return profileName
    
class ManageBookListController():
    def __init__(self, model, view):
        try:
            from PyQt4 import QtCore
        except ImportError:
            from PyQt5 import Qt as QtCore
            
        try:
            from PyQt4.Qt import QComboBox, QLabel
        except ImportError:
            from PyQt5.Qt import QComboBox, QLabel
        
        self.model = model
        self.view = view
        
        profileNames = self.model.keys()
        
        if len(profileNames) > 1:
            profileNames.sort(key=lambda profileName: self.model[profileName][CONFIG_POS_KEY])
            
            self.view.profileLabel.setText('<b>' + CONFIG_LABEL_CHOOSE_PROFILE + '</b>')
            self.view.profileValue = QComboBox()
            
            for profileName in profileNames:
                self.view.profileValue.addItem(profileName)
                
            self.view.profileValue.activated[unicode].connect(self.onProfileActivated)
        else:
            self.view.profileLabel.setText('<b>' + CONFIG_LABEL_PROFILE + '</b>')
            self.view.profileValue = QLabel(profileNames[0])
            
        self.view.dialogLayout.addWidget(self.view.profileValue, 1, 1, 1, 1, QtCore.Qt.AlignLeft)
            
        self.view.buttonBox.accepted.connect(self.view.accept)
        self.view.buttonBox.rejected.connect(self.view.reject)
        
        self.setProfile(profileNames[0])
    
    def onProfileActivated(self, text):
        self.setProfile(getUnicode(text))
    
    def setProfile(self, profileName):
        profile = self.model[profileName]
        booksValue = profile[CONFIG_BOOKS_KEY]
        strategyValue = profile[CONFIG_STRATEGY_KEY]
        orderValue = profile[CONFIG_ORDER_KEY]
        
        details = '<ul>'
        
        if CONFIG_BOOKS_VALUE_SELECTION in booksValue:
            details = details + '<li>' + CONFIG_LABEL_SELECTION + '</li>'
        elif CONFIG_BOOKS_QUERY_KEY in booksValue.keys():
            details = details + '<li>' + CONFIG_LABEL_QUERY
            details = details + '"' + booksValue[CONFIG_BOOKS_QUERY_KEY] + '"'
            details = details + '</li>'
        else:
            details = details + '<li>' + CONFIG_LABEL_COLLECTIONS
            for index, value in enumerate(booksValue[CONFIG_BOOKS_COLLECTIONS_KEY]):
                if index is not 0:
                    details = details + ', '
                details = details + '"' + value + '"'
            details = details + '</li>'
            
        if CONFIG_STRATEGY_VALUE_ONE in strategyValue:
            details = details + '<li>' + CONFIG_LABEL_CHOOSE_ONE + '</li>'
            
        if CONFIG_STRATEGY_VALUE_NEWEST in strategyValue:
            details = details + '<li>' + CONFIG_LABEL_CHOOSE_NEWEST + '</li>'
        elif CONFIG_STRATEGY_VALUE_OLDEST in strategyValue:
            details = details + '<li>' + CONFIG_LABEL_CHOOSE_OLDEST + '</li>'
        else:
            details = details + '<li>' + CONFIG_LABEL_CHOOSE_RANDOM + '</li>'
            
        if CONFIG_ORDER_VALUE_DATE in orderValue:
            details = details + '<li>' + CONFIG_LABEL_ORDER_DATE + '</li>'
        elif CONFIG_ORDER_VALUE_AUTHOR in orderValue:
            details = details + '<li>' + CONFIG_LABEL_ORDER_AUTHOR + '</li>'
        else:
            details = details + '<li>' + CONFIG_LABEL_ORDER_TITLE + '</li>'
            
        details = details + '</ul>'
        
        self.view.profileDetails.setText(details)
        self.view.resize(self.view.sizeHint())
 
class ManageUnreadListView(QDialog):
    def __init__(self, parent):
        try:
            from PyQt4 import QtCore
        except ImportError:
            from PyQt5 import Qt as QtCore
            
        try:
            from PyQt4.Qt import QGridLayout, QLabel, QDialogButtonBox
        except ImportError:
            from PyQt5.Qt import QGridLayout, QLabel, QDialogButtonBox
        
        QDialog.__init__(self, parent)
        
        self.setWindowTitle(CONFIG_LABEL_UNREAD_LIST_SELECT_TITLE)
        
        self.dialogLayout = QGridLayout(self)
        
        self.setUnreadListText = QLabel(CONFIG_LABEL_UNREAD_LIST)
        self.setUnreadListText.setWordWrap(True)
        self.dialogLayout.addWidget(self.setUnreadListText, 0, 0, 1, 2)

        self.profileLabel = QLabel()
        self.dialogLayout.addWidget(self.profileLabel, 1, 0, 1, 1, QtCore.Qt.AlignLeft)
                
        self.infoLabel = QLabel('<b>' + CONFIG_LABEL_DETAILS + '</b>')
        self.dialogLayout.addWidget(self.infoLabel, 2, 0, 1, 2, QtCore.Qt.AlignLeft | QtCore.Qt.AlignTop)

        self.profileDetails = QLabel()
        self.profileDetails.setWordWrap(True)
        self.dialogLayout.addWidget(self.profileDetails, 3, 0, 1, 2)
        
        self.buttonBox = QDialogButtonBox(QDialogButtonBox.Ok | QDialogButtonBox.Cancel)
        self.dialogLayout.addWidget(self.buttonBox, 4, 0, 1, 2)
        
        self.dialogLayout.setColumnStretch(1, 1)
        self.dialogLayout.setColumnMinimumWidth(1, 200)
    
    def getProfileName(self):
        try:
            from PyQt4.Qt import QComboBox
        except ImportError:
            from PyQt5.Qt import QComboBox
        
        if isinstance(self.profileValue, QComboBox):
            profileName = getUnicode(self.profileValue.itemText(self.profileValue.currentIndex()))
        else:
            profileName = getUnicode(self.profileValue.text())
            
        return profileName
    
class ManageUnreadListController():
    def __init__(self, model, view):
        try:
            from PyQt4 import QtCore
        except ImportError:
            from PyQt5 import Qt as QtCore
            
        try:
            from PyQt4.Qt import QComboBox, QLabel
        except ImportError:
            from PyQt5.Qt import QComboBox, QLabel
        
        self.model = model
        self.view = view
        
        profileNames = self.model.keys()
        
        if len(profileNames) > 1:
            profileNames.sort(key=lambda profileName: self.model[profileName][CONFIG_POS_KEY])
            
            self.view.profileLabel.setText('<b>' + CONFIG_LABEL_CHOOSE_PROFILE + '</b>')
            self.view.profileValue = QComboBox()
            
            for profileName in profileNames:
                self.view.profileValue.addItem(profileName)
                
            self.view.profileValue.activated[unicode].connect(self.onProfileActivated)
        else:
            self.view.profileLabel.setText('<b>' + CONFIG_LABEL_PROFILE + '</b>')
            self.view.profileValue = QLabel(profileNames[0])
            
        self.view.dialogLayout.addWidget(self.view.profileValue, 1, 1, 1, 1, QtCore.Qt.AlignLeft)
            
        self.view.buttonBox.accepted.connect(self.accept)
        self.view.buttonBox.rejected.connect(self.view.reject)
        
        self.setProfile(profileNames[0])
    
    def accept(self):
        from calibre_plugins.ManageSonyX50BookList.action import prefs
        
        accepted = True
        
        if prefs[CONFIG_UNREAD_LIST_KEY][CONFIG_SHOW_WARNING_KEY]:
            dialog = SetUnreadListWarningView(self.view)
            controller = SetUnreadListWarningController(dialog)
        
            if dialog.exec_() != dialog.Accepted:
                accepted = False
        
        if accepted is False:
            self.view.reject()
        else:
            self.view.accept()
    
    def onProfileActivated(self, text):
        self.setProfile(getUnicode(text))
    
    def setProfile(self, profileName):
        profile = self.model[profileName]
        booksValue = profile[CONFIG_BOOKS_KEY]
        
        details = '<ul>'
        
        if CONFIG_BOOKS_QUERY_KEY in booksValue.keys():
            details = details + '<li>' + CONFIG_LABEL_QUERY
            details = details + '"' + booksValue[CONFIG_BOOKS_QUERY_KEY] + '"'
            details = details + '</li>'
        else:
            details = details + '<li>' + CONFIG_LABEL_COLLECTIONS
            for index, value in enumerate(booksValue[CONFIG_BOOKS_COLLECTIONS_KEY]):
                if index is not 0:
                    details = details + ', '
                details = details + '"' + value + '"'
            details = details + '</li>'
            
        details = details + '</ul>'
        
        self.view.profileDetails.setText(details)
        self.view.resize(self.view.sizeHint())
 
class SetUnreadListWarningView(QDialog):
    def __init__(self, parent):
        try:
            from PyQt4 import QtCore
        except ImportError:
            from PyQt5 import Qt as QtCore
            
        try:
            from PyQt4.Qt import QGridLayout, QLabel, QCheckBox, QDialogButtonBox
        except ImportError:
            from PyQt5.Qt import QGridLayout, QLabel, QCheckBox, QDialogButtonBox
        
        QDialog.__init__(self, parent)
        
        self.setWindowTitle(CONFIG_LABEL_UNREAD_WARNING_TITLE)
        self.setWindowIcon(createIcon(iconName=ICON_WARNING))
        
        self.dialogLayout = QGridLayout(self)
        
        self.warningIcon = QLabel()
        self.warningIcon.setPixmap(createPixmap(iconName=ICON_WARNING))
        self.dialogLayout.addWidget(self.warningIcon, 0, 0, 1, 1)
        
        self.warningText = QLabel(CONFIG_LABEL_UNREAD_WARNING)
        self.warningText.setWordWrap(True)
        self.dialogLayout.addWidget(self.warningText, 0, 1, 1, 1)
        
        self.showWarning = QCheckBox(CONFIG_LABEL_ALWAYS_SHOW_WARNING)
        self.dialogLayout.addWidget(self.showWarning, 1, 0, 1, 2)

        self.buttonBox = QDialogButtonBox(QDialogButtonBox.Ok | QDialogButtonBox.Cancel)
        self.dialogLayout.addWidget(self.buttonBox, 2, 0, 1, 2)
        
        self.dialogLayout.setColumnStretch(1, 1)
    
class SetUnreadListWarningController():
    def __init__(self, view):
        self.view = view
        
        self.view.showWarning.setChecked(True)
            
        self.view.buttonBox.accepted.connect(self.accept)
        self.view.buttonBox.rejected.connect(self.view.reject)
    
    def accept(self):
        from calibre_plugins.ManageSonyX50BookList.action import prefs
        
        if self.view.showWarning.isChecked() is False:
            prefs[CONFIG_UNREAD_LIST_KEY][CONFIG_SHOW_WARNING_KEY] = False
            prefs.commit()
        
        self.view.accept()

class ManageBookListConfigView(QDialog):
    def __init__(self):
        try:
            from PyQt4 import QtCore
        except ImportError:
            from PyQt5 import Qt as QtCore
            
        try:
            from PyQt4.Qt import QGridLayout, QTabWidget
        except ImportError:
            from PyQt5.Qt import QGridLayout, QTabWidget
        
        QDialog.__init__(self)
        
        self.dialogLayout = QGridLayout(self)
        
        self.tabWidget = QTabWidget(self)
        
        self.booksListConfig = ManageBooksListConfigView()
        self.unreadListConfig = ManageUnreadListConfigView()
        
        self.tabWidget.addTab(self.booksListConfig, CONFIG_LABEL_BOOKS_LIST_TAB)
        self.tabWidget.addTab(self.unreadListConfig, CONFIG_LABEL_UNREAD_LIST_TAB)
        
        self.dialogLayout.addWidget(self.tabWidget, 0, 0, 1, 1)
        self.controller = None
    
    def setController(self, controller):
        self.controller = controller
             
class ManageBookListConfigController():
    def __init__(self, model, view, modelNormalizer):
        self.model = model
        self.view = view
        
        self.booksListConfigController = ManageBooksListConfigController(self.model[CONFIG_BOOKS_LIST_KEY], self.view.booksListConfig, modelNormalizer.normalizeBooksListProfile)
        self.unreadListConfigController = ManageUnreadListConfigController(self.model[CONFIG_UNREAD_LIST_KEY], self.view.unreadListConfig, modelNormalizer.normalizeUnreadListProfile)
        
        self.view.tabWidget.setCurrentIndex(0)
        
    def getBooksListConfig(self):
        return self.booksListConfigController.getBooksListConfig()
    
    def getUnreadListConfig(self):
        return self.unreadListConfigController.getUnreadListConfig()

try:
    from PyQt4.Qt import QWidget
except ImportError:
    from PyQt5.Qt import QWidget

class ManageBooksListConfigView(QWidget):
    def __init__(self):
        try:
            from PyQt4 import QtCore
        except ImportError:
            from PyQt5 import Qt as QtCore
            
        try:
            from PyQt4.Qt import QLabel, QGridLayout, QGroupBox, QCheckBox, QRadioButton, QVBoxLayout
        except ImportError:
            from PyQt5.Qt import QLabel, QGridLayout, QGroupBox, QCheckBox, QRadioButton, QVBoxLayout
        
        QWidget.__init__(self)
        
        self.booksListLayout = QGridLayout(self)
        
        self.setBookListText = QLabel(CONFIG_LABEL_BOOKS_LIST)
        self.setBookListText.setWordWrap(True)
        self.booksListLayout.addWidget(self.setBookListText, 0, 0, 1, 3)
        
        self.profileGroup = ProfileListView()
        self.booksListLayout.addWidget(self.profileGroup, 1, 0, 2, 1)
        
        self.chooseGroup = BookChoiceCriteriaView()
        self.booksListLayout.addWidget(self.chooseGroup, 1, 1, 2, 1)
        
        self.strategyGroup = QGroupBox(CONFIG_LABEL_STRATEGY_GROUP)
        self.oneOption = QCheckBox(CONFIG_LABEL_ONE_OPTION)
        self.newestOption = QRadioButton(CONFIG_LABEL_NEWEST_OPTION)
        self.oldestOption = QRadioButton(CONFIG_LABEL_OLDEST_OPTION)
        self.randomOption = QRadioButton(CONFIG_LABEL_RANDOM_OPTION)
        self.strategyLayout = QVBoxLayout(self.strategyGroup)
        self.strategyLayout.addWidget(self.oneOption)
        self.strategyLayout.addWidget(self.newestOption)
        self.strategyLayout.addWidget(self.oldestOption)
        self.strategyLayout.addWidget(self.randomOption, 1, QtCore.Qt.AlignTop)
        self.strategyGroup.setLayout(self.strategyLayout)
        self.booksListLayout.addWidget(self.strategyGroup, 1, 2, 1, 1)
        
        self.orderGroup = QGroupBox(CONFIG_LABEL_ORDER_GROUP)
        self.dateOption = QRadioButton(CONFIG_LABEL_DATE_OPTION)
        self.authorOption = QRadioButton(CONFIG_LABEL_AUTHOR_OPTION)
        self.titleOption = QRadioButton(CONFIG_LABEL_TITLE_OPTION)
        self.orderLayout = QVBoxLayout(self.orderGroup)
        self.orderLayout.addWidget(self.dateOption)
        self.orderLayout.addWidget(self.authorOption)
        self.orderLayout.addWidget(self.titleOption, 1, QtCore.Qt.AlignTop)
        self.orderGroup.setLayout(self.orderLayout)
        self.booksListLayout.addWidget(self.orderGroup, 2, 2, 1, 1)
        
        self.booksListLayout.setColumnStretch(0, 1)
        self.booksListLayout.setRowStretch(1, 1)
        self.booksListLayout.setRowStretch(2, 1)
        
class ManageBooksListConfigController():
    def __init__(self, model, view, normalizeFunc):
        try:
            from PyQt4 import QtCore
        except ImportError:
            from PyQt5 import Qt as QtCore
        
        self.model = model
        self.view = view
        
        profiles = self.model[CONFIG_PROFILES_KEY]
        self.profileListController = ProfileListController(profiles, view.profileGroup, normalizeFunc)
        self.bookChoiceCriteriaController = BookChoiceCriteriaController(profiles, view.chooseGroup, DEFAULT_CONFIG_BOOKS_LIST_QUERY)
        
        self.view.oneOption.toggled.connect(self.onOneButtonToggled)
        self.view.newestOption.toggled.connect(self.onNewestButtonToggled)
        self.view.oldestOption.toggled.connect(self.onOldestButtonToggled)
        self.view.randomOption.toggled.connect(self.onRandomButtonToggled)
        self.view.dateOption.toggled.connect(self.onDateButtonToggled)
        self.view.authorOption.toggled.connect(self.onAuthorButtonToggled)
        self.view.titleOption.toggled.connect(self.onTitleButtonToggled)
        
        self.profileListController.currentProfileChanged.connect(self.setCurrentProfile)
        self.profileListController.currentProfileChanged.connect(self.bookChoiceCriteriaController.setCurrentProfile)
        
        self.profileListController.initialize()
        
    def getBooksListConfig(self):
        profiles = self.model[CONFIG_PROFILES_KEY]
        
        for index in range(self.view.profileGroup.profileList.count()):
            item = self.view.profileGroup.profileList.item(index)
            profile = profiles[getUnicode(item.text())]
            profile[CONFIG_POS_KEY] = index
        
        return self.model
    
    def getCurrentProfile(self):
        return self.profileListController.getCurrentProfile()
    
    def setCurrentProfile(self, oldProfileName, newProfileName):
        profiles = self.model[CONFIG_PROFILES_KEY]
        newProfile = profiles[getUnicode(newProfileName)]
        strategyValue = newProfile[CONFIG_STRATEGY_KEY]
        orderValue = newProfile[CONFIG_ORDER_KEY]

        self.view.oneOption.setChecked(CONFIG_STRATEGY_VALUE_ONE in strategyValue)
        
        if CONFIG_STRATEGY_VALUE_NEWEST in strategyValue:
            self.view.newestOption.setChecked(True)
        elif CONFIG_STRATEGY_VALUE_OLDEST in strategyValue:
            self.view.oldestOption.setChecked(True)
        else:
            self.view.randomOption.setChecked(True)
        
        if CONFIG_ORDER_VALUE_DATE in orderValue:
            self.view.dateOption.setChecked(True)
        elif CONFIG_ORDER_VALUE_AUTHOR in orderValue:
            self.view.authorOption.setChecked(True)
        else:
            self.view.titleOption.setChecked(True)
    
    def onOneButtonToggled(self, on):
        profile = self.getCurrentProfile()
            
        if on:
            if CONFIG_STRATEGY_VALUE_ONE not in profile[CONFIG_STRATEGY_KEY]:
                profile[CONFIG_STRATEGY_KEY].insert(0, CONFIG_STRATEGY_VALUE_ONE)
        else:
            if CONFIG_STRATEGY_VALUE_ONE in profile[CONFIG_STRATEGY_KEY]:
                profile[CONFIG_STRATEGY_KEY].remove(CONFIG_STRATEGY_VALUE_ONE)
    
    def onNewestButtonToggled(self, on):
        profile = self.getCurrentProfile()
            
        if on:
            if CONFIG_STRATEGY_VALUE_ONE in profile[CONFIG_STRATEGY_KEY]:
                profile[CONFIG_STRATEGY_KEY] = [CONFIG_STRATEGY_VALUE_ONE, CONFIG_STRATEGY_VALUE_NEWEST]
            else:
                profile[CONFIG_STRATEGY_KEY] = [CONFIG_STRATEGY_VALUE_NEWEST]
    
    def onOldestButtonToggled(self, on):
        profile = self.getCurrentProfile()
            
        if on:
            if CONFIG_STRATEGY_VALUE_ONE in profile[CONFIG_STRATEGY_KEY]:
                profile[CONFIG_STRATEGY_KEY] = [CONFIG_STRATEGY_VALUE_ONE, CONFIG_STRATEGY_VALUE_OLDEST]
            else:
                profile[CONFIG_STRATEGY_KEY] = [CONFIG_STRATEGY_VALUE_OLDEST]
    
    def onRandomButtonToggled(self, on):
        profile = self.getCurrentProfile()
            
        if on:
            if CONFIG_STRATEGY_VALUE_ONE in profile[CONFIG_STRATEGY_KEY]:
                profile[CONFIG_STRATEGY_KEY] = [CONFIG_STRATEGY_VALUE_ONE, CONFIG_STRATEGY_VALUE_RANDOM]
            else:
                profile[CONFIG_STRATEGY_KEY] = [CONFIG_STRATEGY_VALUE_RANDOM]
    
    def onDateButtonToggled(self, on):
        profile = self.getCurrentProfile()
            
        if on:
            profile[CONFIG_ORDER_KEY] = CONFIG_ORDER_VALUE_DATE
    
    def onAuthorButtonToggled(self, on):
        profile = self.getCurrentProfile()
            
        if on:
            profile[CONFIG_ORDER_KEY] = CONFIG_ORDER_VALUE_AUTHOR
    
    def onTitleButtonToggled(self, on):
        profile = self.getCurrentProfile()
            
        if on:
            profile[CONFIG_ORDER_KEY] = CONFIG_ORDER_VALUE_TITLE
        
class ManageUnreadListConfigView(QWidget):
    def __init__(self):
        try:
            from PyQt4 import QtCore
        except ImportError:
            from PyQt5 import Qt as QtCore
            
        try:
            from PyQt4.Qt import QLabel, QGridLayout, QCheckBox
        except ImportError:
            from PyQt5.Qt import QLabel, QGridLayout, QCheckBox
        
        QWidget.__init__(self)
        
        self.unreadListLayout = QGridLayout(self)
        
        self.setUnreadListText = QLabel(CONFIG_LABEL_UNREAD_LIST)
        self.setUnreadListText.setWordWrap(True)
        self.unreadListLayout.addWidget(self.setUnreadListText, 0, 0, 1, 2)
        
        self.profileGroup = ProfileListView()
        self.unreadListLayout.addWidget(self.profileGroup, 1, 0, 1, 1)
        
        self.chooseGroup = BookChoiceCriteriaView(False)
        self.unreadListLayout.addWidget(self.chooseGroup, 1, 1, 1, 1)
        
        self.showWarning = QCheckBox(CONFIG_LABEL_SHOW_WARNING)
        self.unreadListLayout.addWidget(self.showWarning, 2, 0, 1, 2)
        
        self.unreadListLayout.setRowStretch(1, 1)
        
class ManageUnreadListConfigController():
    def __init__(self, model, view, normalizeFunc):
        try:
            from PyQt4 import QtCore
        except ImportError:
            from PyQt5 import Qt as QtCore
        
        self.model = model
        self.view = view
        
        self.view.showWarning.setChecked(self.model[CONFIG_SHOW_WARNING_KEY])
        
        profiles = self.model[CONFIG_PROFILES_KEY]
        self.profileListController = ProfileListController(profiles, view.profileGroup, normalizeFunc)
        self.bookChoiceCriteriaController = BookChoiceCriteriaController(profiles, view.chooseGroup, DEFAULT_CONFIG_UNREAD_LIST_QUERY)
        
        self.view.showWarning.toggled.connect(self.onShowWarningToggled)

        self.profileListController.currentProfileChanged.connect(self.bookChoiceCriteriaController.setCurrentProfile)
        
        self.profileListController.initialize()
        
    def getUnreadListConfig(self):
        profiles = self.model[CONFIG_PROFILES_KEY]
        
        for index in range(self.view.profileGroup.profileList.count()):
            item = self.view.profileGroup.profileList.item(index)
            profile = profiles[getUnicode(item.text())]
            profile[CONFIG_POS_KEY] = index
        
        return self.model
        
    def onShowWarningToggled(self, on):
        self.model[CONFIG_SHOW_WARNING_KEY] = on
    
try:
    from PyQt4.Qt import QGroupBox
except ImportError:
    from PyQt5.Qt import QGroupBox

class BookChoiceCriteriaView(QGroupBox):
    def __init__(self, includeSelection=True):
        try:
            from PyQt4 import QtCore
        except ImportError:
            from PyQt5 import Qt as QtCore
            
        try:
            from PyQt4.Qt import QRadioButton, QLineEdit, QListWidget, QPushButton, QGridLayout, QIcon
        except ImportError:
            from PyQt5.Qt import QRadioButton, QLineEdit, QListWidget, QPushButton, QGridLayout, QIcon
        
        QGroupBox.__init__(self)
        
        self.setTitle(CONFIG_LABEL_CHOOSE_GROUP)
        
        if includeSelection:
            self.selectionOption = QRadioButton(CONFIG_LABEL_SELECTION_OPTION)
        else:
            self.selectionOption = None
            
        self.queryOption = QRadioButton(CONFIG_LABEL_QUERY_OPTION)
        self.queryInput = QLineEdit()
        
        self.collectionsOption = QRadioButton(CONFIG_LABEL_COLLECTIONS_OPTION)
        self.collectionsList = QListWidget()
        self.collectionAddButton = QPushButton(CONFIG_LABEL_ADD)
        self.collectionDeleteButton = QPushButton(CONFIG_LABEL_DELETE)
        self.collectionUpButton = QPushButton(QIcon(I('arrow-up.png')), '')
        self.collectionDownButton = QPushButton(QIcon(I('arrow-down.png')), '')
        
        self.chooseLayout = QGridLayout(self)
        
        if self.selectionOption is not None:
            self.chooseLayout.addWidget(self.selectionOption, 0, 0, 1, 2)
            
        self.chooseLayout.addWidget(self.queryOption, 1, 0, 1, 2)
        self.chooseLayout.addWidget(self.queryInput, 2, 0, 1, 2)
        self.chooseLayout.addWidget(self.collectionsOption, 3, 0, 1, 2)
        self.chooseLayout.addWidget(self.collectionsList, 4, 0, 2, 2)
        self.chooseLayout.addWidget(self.collectionUpButton, 4, 2, 1, 1, QtCore.Qt.AlignBottom)
        self.chooseLayout.addWidget(self.collectionDownButton, 5, 2, 1, 1, QtCore.Qt.AlignTop)
        self.chooseLayout.addWidget(self.collectionAddButton, 6, 0, 1, 1)
        self.chooseLayout.addWidget(self.collectionDeleteButton, 6, 1, 1, 1)
             
class BookChoiceCriteriaController():
    def __init__(self, model, view, defaultQuery):
        try:
            from PyQt4 import QtCore
        except ImportError:
            from PyQt5 import Qt as QtCore
        
        self.model = model
        self.view = view
        self.defaultQuery = defaultQuery
        self.currentProfile = None
        
        if self.view.selectionOption is not None:
            self.view.selectionOption.toggled.connect(self.onSelectionButtonToggled)

        self.view.collectionsList.currentItemChanged.connect(self.onCollectionsCurrentItemChanged)
        self.view.collectionsList.itemDoubleClicked.connect(self.onCollectionsItemDoubleClicked)
        self.view.collectionsList.itemChanged.connect(self.onCollectionsItemChanged)
        self.view.queryOption.toggled.connect(self.onQueryButtonToggled)
        self.view.queryInput.editingFinished.connect(self.onQueryEditingFinished)
        self.view.collectionsOption.toggled.connect(self.onCollectionsButtonToggled)
        self.view.collectionDeleteButton.clicked.connect(self.onCollectionDeleteButtonClicked)
        self.view.collectionAddButton.clicked.connect(self.onCollectionAddButtonClicked)
        self.view.collectionUpButton.clicked.connect(self.onCollectionUpButtonClicked)
        self.view.collectionDownButton.clicked.connect(self.onCollectionDownButtonClicked)
              
    def getCurrentCollection(self):
        return getUnicode(self.view.collectionsList.item(self.view.collectionsList.currentRow()).text())
    
    def setCurrentProfile(self, oldProfileName, newProfileName):
        self.currentProfile = self.model[getUnicode(newProfileName)]
        booksValue = self.currentProfile[CONFIG_BOOKS_KEY]

        if CONFIG_BOOKS_VALUE_SELECTION in booksValue:
            self.view.selectionOption.setChecked(True)
        elif CONFIG_BOOKS_QUERY_KEY in booksValue.keys():
            if self.view.queryOption.isChecked():
                self.onQueryButtonToggled(True)
            else:
                self.view.queryOption.setChecked(True)
        else:
            if self.view.collectionsOption.isChecked():
                self.onCollectionsButtonToggled(True)
            else:
                self.view.collectionsOption.setChecked(True)
            
    def onCollectionsCurrentItemChanged(self, newItem, previousItem):
        if newItem is not None:
            self.view.collectionDeleteButton.setEnabled(self.view.collectionsList.count() > 1)
            self.view.collectionUpButton.setEnabled(self.view.collectionsList.count() > 1 and self.view.collectionsList.currentRow() > 0)
            self.view.collectionDownButton.setEnabled(self.view.collectionsList.count() > 1 and self.view.collectionsList.currentRow() < self.view.collectionsList.count() - 1)
    
    def onCollectionsItemDoubleClicked(self, item):
        self.view.collectionsList.editItem(item)
    
    def onSelectionButtonToggled(self, on):
        if on:
            self.currentProfile[CONFIG_BOOKS_KEY] = CONFIG_BOOKS_VALUE_SELECTION

            self.view.queryInput.clear()
            self.view.queryInput.setDisabled(True)
            
            self.view.collectionAddButton.setDisabled(True)
            self.view.collectionDeleteButton.setDisabled(True)
            self.view.collectionUpButton.setDisabled(True)
            self.view.collectionDownButton.setDisabled(True)
            self.view.collectionsList.clear()
            self.view.collectionsList.setDisabled(True)
    
    def onQueryButtonToggled(self, on):
        if on:
            booksValue = self.currentProfile[CONFIG_BOOKS_KEY]

            if (CONFIG_BOOKS_VALUE_SELECTION in booksValue) or (CONFIG_BOOKS_COLLECTIONS_KEY in booksValue.keys()):
                self.currentProfile[CONFIG_BOOKS_KEY] = {CONFIG_BOOKS_QUERY_KEY: self.defaultQuery}
                booksValue = self.currentProfile[CONFIG_BOOKS_KEY]
            
            self.view.queryInput.setDisabled(False)
            self.view.queryInput.setText(booksValue[CONFIG_BOOKS_QUERY_KEY])
                
            self.view.collectionAddButton.setDisabled(True)
            self.view.collectionDeleteButton.setDisabled(True)
            self.view.collectionUpButton.setDisabled(True)
            self.view.collectionDownButton.setDisabled(True)
            self.view.collectionsList.clear()
            self.view.collectionsList.setDisabled(True)
                
    def onCollectionsButtonToggled(self, on):
        try:
            from PyQt4 import QtCore
        except ImportError:
            from PyQt5 import Qt as QtCore
        
        if on:
            autoEdit = False
            booksValue = self.currentProfile[CONFIG_BOOKS_KEY]
            
            if (CONFIG_BOOKS_VALUE_SELECTION in booksValue) or (CONFIG_BOOKS_QUERY_KEY in booksValue.keys()):
                self.currentProfile[CONFIG_BOOKS_KEY] = {CONFIG_BOOKS_COLLECTIONS_KEY: [DEFAULT_COLLECTION_NAME]}
                booksValue = self.currentProfile[CONFIG_BOOKS_KEY]
                autoEdit = True
            
            self.view.collectionDeleteButton.setDisabled(True)
            self.view.collectionUpButton.setDisabled(True)
            self.view.collectionDownButton.setDisabled(True)
            self.view.collectionsList.clear()
            self.view.collectionsList.setDisabled(False)
            
            for collection in booksValue[CONFIG_BOOKS_COLLECTIONS_KEY]:
                self.view.collectionsList.addItem(collection)
                item = self.view.collectionsList.item(self.view.collectionsList.count() - 1)
                item.setFlags(item.flags() | QtCore.Qt.ItemIsEditable)
                
            if autoEdit:
                self.view.collectionsList.editItem(self.view.collectionsList.item(0))
                
            self.view.collectionAddButton.setEnabled(True)
    
            self.view.queryInput.clear()
            self.view.queryInput.setDisabled(True)
    
    def onQueryEditingFinished(self):
        queryString = getUnicode(self.view.queryInput.text()).strip()
        
        if len(queryString) is 0:
            queryString = self.defaultQuery
            self.view.queryInput.setText(queryString)
        
        self.currentProfile[CONFIG_BOOKS_KEY][CONFIG_BOOKS_QUERY_KEY] = queryString
            
    def onCollectionDeleteButtonClicked(self):
        currentRow = self.view.collectionsList.currentRow()
        previousCollection = self.view.collectionsList.takeItem(currentRow)
        newCollection = self.view.collectionsList.item(self.view.collectionsList.currentRow())
        self.onCollectionsCurrentItemChanged(newCollection, previousCollection)
        
        collections = self.currentProfile[CONFIG_BOOKS_KEY][CONFIG_BOOKS_COLLECTIONS_KEY]
        collections.remove(getUnicode(previousCollection.text()))
        
        del previousCollection
    
    def onCollectionUpButtonClicked(self):
        currentRow = self.view.collectionsList.currentRow()
        collectionItem = self.view.collectionsList.takeItem(currentRow)
        self.view.collectionsList.insertItem(currentRow - 1, collectionItem)
        self.view.collectionsList.setCurrentRow(currentRow - 1)
        
        collection = self.getCurrentCollection()
        collections = self.currentProfile[CONFIG_BOOKS_KEY][CONFIG_BOOKS_COLLECTIONS_KEY]
        index = collections.index(collection)
        collections.pop(index)
        collections.insert(index - 1, collection)
    
    def onCollectionDownButtonClicked(self):
        currentRow = self.view.collectionsList.currentRow()
        collectionItem = self.view.collectionsList.takeItem(currentRow)
        self.view.collectionsList.insertItem(currentRow + 1, collectionItem)
        self.view.collectionsList.setCurrentRow(currentRow + 1)
        
        collection = self.getCurrentCollection()
        collections = self.currentProfile[CONFIG_BOOKS_KEY][CONFIG_BOOKS_COLLECTIONS_KEY]
        index = collections.index(collection)
        collections.pop(index)
        collections.insert(index + 1, collection)
    
    def onCollectionAddButtonClicked(self):
        try:
            from PyQt4 import QtCore
        except ImportError:
            from PyQt5 import Qt as QtCore
        
        collections = self.currentProfile[CONFIG_BOOKS_KEY][CONFIG_BOOKS_COLLECTIONS_KEY]
        name = getUniqueName(DEFAULT_COLLECTION_NAME, collections)
        collections.append(name)
        
        self.view.collectionsList.addItem(name)
        lastRow = self.view.collectionsList.count() - 1
        self.view.collectionsList.setCurrentRow(lastRow)
        item = self.view.collectionsList.item(lastRow)
        item.setFlags(item.flags() | QtCore.Qt.ItemIsEditable)
        self.view.collectionsList.editItem(item)
    
    def onCollectionsItemChanged(self, item):
        collections = self.currentProfile[CONFIG_BOOKS_KEY][CONFIG_BOOKS_COLLECTIONS_KEY]
        row = self.view.collectionsList.row(item)
        oldName = collections[row]
        newName = getUnicode(item.text()).strip()
        
        if newName != getUnicode(item.text()): # Ignore added leading and trailing spaces
            item.setText(newName)
        
        if newName is None or len(newName) is 0:
            newName = oldName
            item.setText(oldName)
        
        if newName != oldName:
            uniqueName = getUniqueName(newName, collections)
            collections[row] = uniqueName
            item.setText(uniqueName)
    
class ProfileListView(QGroupBox):
    def __init__(self):
        try:
            from PyQt4 import QtCore
        except ImportError:
            from PyQt5 import Qt as QtCore
            
        try:
            from PyQt4.Qt import QListWidget, QPushButton, QGridLayout, QIcon
        except ImportError:
            from PyQt5.Qt import QListWidget, QPushButton, QGridLayout, QIcon
        
        QGroupBox.__init__(self)
        
        self.setTitle(CONFIG_LABEL_PROFILES)
        
        self.profileList = QListWidget()
        self.profileAddButton = QPushButton(CONFIG_LABEL_ADD)
        self.profileDeleteButton = QPushButton(CONFIG_LABEL_DELETE)
        self.profileUpButton = QPushButton(QIcon(I('arrow-up.png')), '')
        self.profileDownButton = QPushButton(QIcon(I('arrow-down.png')), '')
        self.profileLayout = QGridLayout(self)
        self.profileLayout.addWidget(self.profileList, 0, 0, 2, 2)
        self.profileLayout.addWidget(self.profileAddButton, 2, 0, 1, 1)
        self.profileLayout.addWidget(self.profileDeleteButton, 2, 1, 1, 1)
        self.profileLayout.addWidget(self.profileUpButton, 0, 2, 1, 1, QtCore.Qt.AlignBottom)
        self.profileLayout.addWidget(self.profileDownButton, 1, 2, 1, 1, QtCore.Qt.AlignTop)
    
try:
    from PyQt4.QtCore import QObject, pyqtSignal
except ImportError:
    from PyQt5.QtCore import QObject, pyqtSignal
             
class ProfileListController(QObject):
    currentProfileChanged = pyqtSignal(unicode, unicode)
    
    def __init__(self, model, view, normalizeFunc):
        try:
            from PyQt4 import QtCore
        except ImportError:
            from PyQt5 import Qt as QtCore
        
        QObject.__init__(self)
        
        self.model = model
        self.normalizeFunc = normalizeFunc
        self.view = view
        
        self.currentModelProfile = None
    
        profiles = self.model
        profileNames = profiles.keys()
        profileNames.sort(key=lambda profileName: profiles[profileName][CONFIG_POS_KEY])
        
        for profileName in profileNames:
            self.view.profileList.addItem(profileName)
            item = self.view.profileList.item(self.view.profileList.count() - 1)
            item.setFlags(item.flags() | QtCore.Qt.ItemIsEditable)
        
        self.view.profileList.currentItemChanged.connect(self.onProfileCurrentItemChanged)
        self.view.profileList.itemDoubleClicked.connect(self.onProfileItemDoubleClicked)
        self.view.profileList.itemChanged.connect(self.onProfileItemChanged)
        self.view.profileDeleteButton.clicked.connect(self.onProfileDeleteButtonClicked)
        self.view.profileAddButton.clicked.connect(self.onProfileAddButtonClicked)
        self.view.profileUpButton.clicked.connect(self.onProfileUpButtonClicked)
        self.view.profileDownButton.clicked.connect(self.onProfileDownButtonClicked)
        
    def initialize(self):
        self.view.profileList.setCurrentRow(0)
            
    def getCurrentProfile(self):
        profileName = getUnicode(self.view.profileList.item(self.view.profileList.currentRow()).text())
        
        return self.model[profileName]
    
    def setCurrentProfile(self, oldProfileName, newProfileName):
        self.currentModelProfile = self.model[newProfileName]
            
        self.currentProfileChanged.emit(oldProfileName, newProfileName)
    
    def onProfileCurrentItemChanged(self, newItem, previousItem):
        if newItem is not None:
            if previousItem is None:
                oldProfileName = getUnicode('')
            else:
                oldProfileName = getUnicode(previousItem.text())
                
            newProfileName = getUnicode(newItem.text())
                
            self.setCurrentProfile(oldProfileName, newProfileName)
            
            self.view.profileDeleteButton.setEnabled(self.view.profileList.count() > 1)
            self.view.profileUpButton.setEnabled(self.view.profileList.count() > 1 and self.view.profileList.currentRow() > 0)
            self.view.profileDownButton.setEnabled(self.view.profileList.count() > 1 and self.view.profileList.currentRow() < self.view.profileList.count() - 1)
    
    def onProfileItemDoubleClicked(self, item):
        self.view.profileList.editItem(item)
    
    def onProfileDeleteButtonClicked(self):
        profile = self.getCurrentProfile()
        currentRow = self.view.profileList.currentRow()
        previousProfile = self.view.profileList.takeItem(currentRow)
        newProfile = self.view.profileList.item(self.view.profileList.currentRow())
        self.onProfileCurrentItemChanged(newProfile, previousProfile)
        
        del self.model[getUnicode(previousProfile.text())]
        
        del previousProfile
    
    def onProfileUpButtonClicked(self):
        currentRow = self.view.profileList.currentRow()
        profileItem = self.view.profileList.takeItem(currentRow)
        self.view.profileList.insertItem(currentRow - 1, profileItem)
        self.view.profileList.setCurrentRow(currentRow - 1)
    
    def onProfileDownButtonClicked(self):
        currentRow = self.view.profileList.currentRow()
        profileItem = self.view.profileList.takeItem(currentRow)
        self.view.profileList.insertItem(currentRow + 1, profileItem)
        self.view.profileList.setCurrentRow(currentRow + 1)
    
    def onProfileAddButtonClicked(self):
        try:
            from PyQt4 import QtCore
        except ImportError:
            from PyQt5 import Qt as QtCore
        
        profiles = self.model
        name = getUniqueName(DEFAULT_PROFILE_NAME, profiles.keys())
        profile = {}
        self.normalizeFunc(name, profile)
        profiles[name] = profile
        
        self.view.profileList.addItem(name)
        lastRow = self.view.profileList.count() - 1
        self.view.profileList.setCurrentRow(lastRow)
        item = self.view.profileList.item(lastRow)
        item.setFlags(item.flags() | QtCore.Qt.ItemIsEditable)
        self.view.profileList.editItem(item)
    
    def onProfileItemChanged(self, item):
        import copy
        
        profile = self.currentModelProfile
        oldName = profile[CONFIG_NAME_KEY]
        newName = getUnicode(item.text()).strip()
        
        if newName != getUnicode(item.text()): # Ignore added leading and trailing spaces
            item.setText(newName)
        
        if newName is None or len(newName) is 0:
            newName = oldName
            item.setText(oldName)
        
        if newName != oldName:
            profiles = self.model
            uniqueName = getUniqueName(newName, profiles.keys())
            newProfile = copy.copy(profile)
            self.normalizeFunc(uniqueName, newProfile)
            profiles[uniqueName] = newProfile
            self.setCurrentProfile(oldName, uniqueName)
            del profiles[oldName]
            item.setText(uniqueName)
        